/*
 *  xfmedia - simple gtk2 media player based on xine
 *
 *  Copyright (c) 2004 Brian Tarricone, <bjt23@cornell.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include <X11/Xlib.h>

#include <gmodule.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>

#include <libxfce4util/libxfce4util.h>
#include <libxfcegui4/libxfcegui4.h>

#include "xfmedia-common.h"
#include "xfmedia-icon.h"

#if !GTK_CHECK_VERSION(2, 6, 0)
#include "xfmedia-stock-icons.h"
#endif

XfceIconTheme *icon_theme = NULL;

static gchar *
convert_filename(const gchar *filename)
{
    GString *gstr;
    gchar *newfilename, *c;
    
    gstr = g_string_sized_new(strlen(filename)+20);
    
    for(c = (gchar *)filename; *c; c++) {
        switch(*c) {
            case '_':
                g_string_append(gstr, " ");
                break;
            case '%':
                c++;
                if(*c == '2') {
                    c++;
                    if(*c == '0')
                        g_string_append(gstr, " ");
                    else
                        g_string_append_printf(gstr, "%%2%c", *c);
                } else
                    g_string_append_printf(gstr, "%%%c", *c);
                break;
            default:
                g_string_append_c(gstr, *c);
                break;
        }
    }
    
    newfilename = gstr->str;
    g_string_free(gstr, FALSE);
    
    return newfilename;
}

GtkWidget *
xfmedia_custom_button_new_from_pixdata(const gchar *text,
        const GdkPixdata *pixdata)
{
    GtkWidget *btn, *img, *hbox, *lbl;
    GdkPixbuf *pix, *tmppix;
    gint iw, ih;
    
    hbox = gtk_hbox_new(FALSE, 4);
    gtk_widget_show(hbox);
    
    gtk_icon_size_lookup(GTK_ICON_SIZE_BUTTON, &iw, &ih);
    pix = gdk_pixbuf_from_pixdata(pixdata, FALSE, NULL);
    ih = gdk_pixbuf_get_height(pix);
    if(iw != ih) {
        tmppix = gdk_pixbuf_scale_simple(pix, iw, iw, GDK_INTERP_BILINEAR);
        g_object_unref(G_OBJECT(pix));
        pix = tmppix;
    }
    img = gtk_image_new_from_pixbuf(pix);
    g_object_unref(G_OBJECT(pix));
    gtk_widget_show(img);
    gtk_box_pack_start(GTK_BOX(hbox), img, FALSE, FALSE, 0);
    
    lbl = gtk_label_new_with_mnemonic(text);
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
    
    btn = gtk_button_new();
    gtk_container_add(GTK_CONTAINER(btn), hbox);
    gtk_label_set_mnemonic_widget(GTK_LABEL(lbl), btn);
    
    return btn;
}

GtkWidget *
xfmedia_custom_button_new(const gchar *text, const gchar *icon)
{
    GtkWidget *btn, *hbox, *img, *lbl;
    GdkPixbuf *pix;
    gint iw, ih;
    
    hbox = gtk_hbox_new(FALSE, 4);
    gtk_widget_show(hbox);
    
    img = gtk_image_new_from_stock(icon, GTK_ICON_SIZE_BUTTON);
    if(!img || gtk_image_get_storage_type(GTK_IMAGE(img)) == GTK_IMAGE_EMPTY) {
        gtk_icon_size_lookup(GTK_ICON_SIZE_BUTTON, &iw, &ih);
        pix = xfce_icon_theme_load(icon_theme, icon, iw);
        if(pix) {
            if(img)
                gtk_image_set_from_pixbuf(GTK_IMAGE(img), pix);
            else
                img = gtk_image_new_from_pixbuf(pix);
            g_object_unref(G_OBJECT(pix));
        }
    }
    if(img) {
        gtk_widget_show(img);
        gtk_box_pack_start(GTK_BOX(hbox), img, FALSE, FALSE, 0);
    }
    
    lbl = gtk_label_new_with_mnemonic(text);
    gtk_widget_show(lbl);
    gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
    
    btn = gtk_button_new();
    gtk_container_add(GTK_CONTAINER(btn), hbox);
    gtk_label_set_mnemonic_widget(GTK_LABEL(lbl), btn);
    
    return btn;
}

gchar *
xfmedia_filename_to_name(const gchar *filename)
{
    gchar *p, *q, *utf8, *tmp;
    const gchar *charset = NULL;
    gint written = 0;
    
    /* FIXME: i don't like this, but i don't want e.g. "dvd://" to return
     * a blank name */
    if((tmp=strstr(filename, ":/")) && tmp-filename < 6 && !strstr(filename, "://"))
        p = g_strdup(filename);
    else {
        p = g_strrstr(filename, G_DIR_SEPARATOR_S);
        if(p)
            p = g_strdup(p+1);
        else
            p = g_strdup(filename);
        q = strstr(p, "_");
        while(q) {
            *q = ' ';
            q = strstr(q+1, "_");
        }
        q = g_strrstr(p, ".");
        if(q)
            *q = 0;
    }
    
    if(!g_utf8_validate(p, -1, NULL)) {
        if(!g_get_charset(&charset)) {
            utf8 = g_convert_with_fallback(p, strlen(p), "UTF-8", charset, "?",
                    NULL, &written, NULL);
        } else
            utf8 = g_locale_to_utf8(p, -1, NULL, &written, NULL);
        if(!utf8)
            return p;
        g_free(p);
        return utf8;
    }
    
    return p;
}

gchar *
xfmedia_sinfo_to_string(xine_stream_t *stream, const gchar *fmt, const gchar *mrl)
{
    GString *gstr;
    gchar *str = NULL, *c, *filename;
    const gchar *value;
    gboolean have_a_field = FALSE;
    
    /*DBG("xfmedia_sinfo_to_string(%p, %s, %s)\n", stream, fmt, mrl);*/
    
    gstr = g_string_sized_new(128);
    
    for(c = (gchar *)fmt; *c; c++) {
        if(*c == '%') {
            c++;
            switch(*c) {
                case 'p':
                    value = xine_get_meta_info(stream, XINE_META_INFO_ARTIST);
                    if(value && *value) {
                        g_string_append(gstr, value);
                        have_a_field = TRUE;
                    }
                    break;
                case 't':
                    value = xine_get_meta_info(stream, XINE_META_INFO_TITLE);
                    if(value && *value) {
                        g_string_append(gstr, value);
                        have_a_field = TRUE;
                    }
                    break;
                case 'a':
                    value = xine_get_meta_info(stream, XINE_META_INFO_ALBUM);
                    if(value && *value) {
                        g_string_append(gstr, value);
                        have_a_field = TRUE;
                    }
                    break;
                case 'T':
                    /* not supported by xine =( */
                    break;
                case 'g':
                    value = xine_get_meta_info(stream, XINE_META_INFO_GENRE);
                    if(value && *value) {
                        g_string_append(gstr, value);
                        have_a_field = TRUE;
                    }
                    break;
                case 'y':
                    value = xine_get_meta_info(stream, XINE_META_INFO_YEAR);
                    if(value && *value) {
                        g_string_append(gstr, value);
                        have_a_field = TRUE;
                    }
                    break;
                case 'f':
                    if(mrl) {
                        gchar *p = g_strrstr(mrl, G_DIR_SEPARATOR_S);
                        if(!p)
                            p = (gchar *)mrl;
                        else
                            p++;
                        filename = convert_filename(p);
                        g_string_append(gstr, filename);
                        g_free(filename);
                        have_a_field = TRUE;
                    }
                    break;
                case '%':
                    g_string_append(gstr, "%");
                    break;
                default:
                    g_string_append_printf(gstr, "%%%c", *c);
                    break;
            }
        } else
            g_string_append_c(gstr, *c);
    }
    
    if(have_a_field) {
        str = gstr->str;
        g_string_free(gstr, FALSE);
    } else
        g_string_free(gstr, TRUE);
    
    return str;
}

void
xfmedia_init_icon_theme()
{
    if(icon_theme)
        return;
    
    icon_theme = xfce_icon_theme_get_for_screen(NULL);
    xfce_icon_theme_prepend_search_path(icon_theme, DATADIR "/icons/");
}

gchar *
xfmedia_num_prettify(const gchar *buf)
{
    gchar *p, *np, *str;
    gint newlen, commas, i;
    
    if(!buf)
        return NULL;
    if(strlen(buf) < 4)
        return g_strdup(buf);
    
    commas = (strlen(buf)-1) / 3;
    newlen = strlen(buf) + commas;
    str = g_malloc0(newlen+1);
    
    p = (gchar *)buf + strlen(buf) - 1;
    np = str + newlen - 1;
    
    i = 3;
    for(;;) {
        if(i-- == 0) {
            *np = ',';
            np--;
            i = 2;
        }
        *np = *p;
        
        if(p-- == buf || np-- == str)
            break;
    }
    
    return str;
}

GdkWindow *
xfmedia_get_window_decorations(GdkWindow *gwindow)
{
    GdkWindow *wmdec_win = NULL;
    GdkDisplay *gdpy = gdk_drawable_get_display(GDK_DRAWABLE(gwindow));
    Display *dpy = GDK_DISPLAY_XDISPLAY(gdpy);
    Window window = GDK_WINDOW_XWINDOW(gwindow);
    Window root_return, parent_return, *children_return = NULL;
    unsigned int nchildren = 0;
    
    XGrabServer(dpy);
    for(;;) {
        root_return = parent_return = None;
        XQueryTree(dpy, window, &root_return, &parent_return,
                &children_return, &nchildren);
        XFree(children_return);
        if(root_return == None || parent_return == None) {
            XUngrabServer(dpy);
            return NULL;
        }
        else if(root_return == parent_return)
            break;
        window = parent_return;
    };
    XUngrabServer(dpy);
    
    wmdec_win = gdk_window_foreign_new_for_display(gdpy, window);
    
    return wmdec_win;
}

/* ripped off of libgnomesomething, i think. */
GList *
xfmedia_uri_list_extract_uris(const gchar *uri_list)
{
    const gchar *p, *q;
    gchar *retval;
    GList *result = NULL;
    
    g_return_val_if_fail(uri_list != NULL, NULL);
    
    p = uri_list;
    
    /* We don't actually try to validate the URI according to RFC
     * 2396, or even check for allowed characters - we just ignore
     * comments and trim whitespace off the ends.  We also
     * allow LF delimination as well as the specified CRLF.
     */
    while(p) {
        if(*p != '#') {
            while(g_ascii_isspace((int)*p))
                p++;
            
            q = p;
            while(*q && (*q != '\n') && (*q != '\r'))
                q++;
            
            if(q > p) {
                q--;
                while(q > p && g_ascii_isspace((int)*q))
                    q--;
                
                retval = (gchar *)g_malloc(q - p + 2);
                strncpy(retval, p, q - p + 1);
                retval[q - p + 1] = '\0';
                
                result = g_list_prepend (result, retval);
            }
        }
        p = strchr (p, '\n');
        if(p)
            p++;
    }
    
    return g_list_reverse(result);
}

gint
xfmedia_tree_path_to_index(GtkTreePath *path)
{
    gchar *idx_str;
    gint idx;
    
    g_return_val_if_fail(path, -1);
    
    idx_str = gtk_tree_path_to_string(path);
    idx = atoi(idx_str);
    g_free(idx_str);
    
    return (idx >= 0 ? idx : -1);
}

gboolean
xfmedia_list_store_remove_path(GtkListStore *ls, GtkTreePath *path)
{
    GtkTreeIter itr;
    
    if(gtk_tree_model_get_iter(GTK_TREE_MODEL(ls), &itr, path))
        return gtk_list_store_remove(ls, &itr);
    return FALSE;
}

/**
 * xfmedia_brutal_utf8_validate:
 * @string: string to validate
 *
 * Tries its damnedest to convert @string into valid UTF-8.  Any string
 * returned is guaranteed to be UTF-8.  If it returns %NULL, then the string
 * passed in was already valid UTF-8.
 *
 * Returns: Valid UTF-8 string (free with g_free() when done), or %NULL if
 *          @string is already valid UTF-8.
 **/
gchar *
xfmedia_brutal_utf8_validate(const gchar *string)
{
    gchar *utf8 = NULL;
    const gchar *charset = NULL;
    gint read = 0, written = 0;
    GError *err = NULL;
    
    if(!g_utf8_validate(string, -1, NULL)) {
        if(!g_get_charset(&charset))
            charset = "ISO-8859-15";  /* better than nothing */
        utf8 = g_convert_with_fallback(string, strlen(string), "UTF-8", charset,
                "?", &read, &written, &err);
        if(!utf8 || err) {
            if(err) {
                g_warning("Xfmedia: Unable to convert string to UTF-8 using default method (%d): %s",
                        err->code, err->message);
                g_error_free(err);
            }
            if(utf8)  /* will this ever happen? */
                g_free(utf8);
            read = written = 0;
            utf8 = g_convert_with_fallback(string, strlen(string), "UTF-8", charset,
                "?", &read, &written, NULL);
            if(!utf8) {
                gchar *tmp = g_strndup(string, read);
                read = written = 0;
                utf8 = g_convert_with_fallback(tmp, strlen(tmp), "UTF-8",
                        charset," ?", &read, &written, NULL);
                g_free(tmp);
                if(!utf8) {
                    g_critical("Xfmedia: Unable to convert string to UTF-8 using last-ditch effort.");
                    utf8 = g_strdup(_("(title conversion failed)"));
                }
            }
        }
    }
    
    return utf8;
}



void
xfmedia_stock_icons_init()
{
#if !GTK_CHECK_VERSION(2, 6, 0)
    GtkIconFactory *ifac;
    GtkIconSource *isrc;
    GtkIconSet *iset;
    GdkPixbuf *pix;
    
    ifac = gtk_icon_factory_new();
    gtk_icon_factory_add_default(ifac);
    
    /* folder icon */
    iset = gtk_icon_set_new();
    isrc = gtk_icon_source_new();
    pix = gdk_pixbuf_from_pixdata(&stock_folder, FALSE, NULL);
    gtk_icon_source_set_pixbuf(isrc, pix);
    g_object_unref(G_OBJECT(pix));
    gtk_icon_source_set_size_wildcarded(isrc, TRUE);
    gtk_icon_source_set_state_wildcarded(isrc, TRUE);
    gtk_icon_source_set_direction_wildcarded(isrc, TRUE);
    gtk_icon_set_add_source(iset, isrc);
    gtk_icon_factory_add(ifac, GTK_STOCK_DIRECTORY, iset);
    gtk_icon_set_unref(iset);
    
    /* media icons */
    
    iset = gtk_icon_set_new();
    isrc = gtk_icon_source_new();
    pix = gdk_pixbuf_from_pixdata(&xfmedia_next, FALSE, NULL);
    gtk_icon_source_set_pixbuf(isrc, pix);
    g_object_unref(G_OBJECT(pix));
    gtk_icon_source_set_size_wildcarded(isrc, TRUE);
    gtk_icon_source_set_state_wildcarded(isrc, TRUE);
    gtk_icon_source_set_direction_wildcarded(isrc, TRUE);
    gtk_icon_set_add_source(iset, isrc);
    gtk_icon_factory_add(ifac, GTK_STOCK_MEDIA_NEXT, iset);
    gtk_icon_set_unref(iset);
    
    iset = gtk_icon_set_new();
    isrc = gtk_icon_source_new();
    pix = gdk_pixbuf_from_pixdata(&xfmedia_pause, FALSE, NULL);
    gtk_icon_source_set_pixbuf(isrc, pix);
    g_object_unref(G_OBJECT(pix));
    gtk_icon_source_set_size_wildcarded(isrc, TRUE);
    gtk_icon_source_set_state_wildcarded(isrc, TRUE);
    gtk_icon_source_set_direction_wildcarded(isrc, TRUE);
    gtk_icon_set_add_source(iset, isrc);
    gtk_icon_factory_add(ifac, GTK_STOCK_MEDIA_PAUSE, iset);
    gtk_icon_set_unref(iset);
    
    iset = gtk_icon_set_new();
    isrc = gtk_icon_source_new();
    pix = gdk_pixbuf_from_pixdata(&xfmedia_play, FALSE, NULL);
    gtk_icon_source_set_pixbuf(isrc, pix);
    g_object_unref(G_OBJECT(pix));
    gtk_icon_source_set_size_wildcarded(isrc, TRUE);
    gtk_icon_source_set_state_wildcarded(isrc, TRUE);
    gtk_icon_source_set_direction_wildcarded(isrc, TRUE);
    gtk_icon_set_add_source(iset, isrc);
    gtk_icon_factory_add(ifac, GTK_STOCK_MEDIA_PLAY, iset);
    gtk_icon_set_unref(iset);
    
    iset = gtk_icon_set_new();
    isrc = gtk_icon_source_new();
    pix = gdk_pixbuf_from_pixdata(&xfmedia_prev, FALSE, NULL);
    gtk_icon_source_set_pixbuf(isrc, pix);
    g_object_unref(G_OBJECT(pix));
    gtk_icon_source_set_size_wildcarded(isrc, TRUE);
    gtk_icon_source_set_state_wildcarded(isrc, TRUE);
    gtk_icon_source_set_direction_wildcarded(isrc, TRUE);
    gtk_icon_set_add_source(iset, isrc);
    gtk_icon_factory_add(ifac, GTK_STOCK_MEDIA_PREVIOUS, iset);
    gtk_icon_set_unref(iset);
    
    iset = gtk_icon_set_new();
    isrc = gtk_icon_source_new();
    pix = gdk_pixbuf_from_pixdata(&xfmedia_stop, FALSE, NULL);
    gtk_icon_source_set_pixbuf(isrc, pix);
    g_object_unref(G_OBJECT(pix));
    gtk_icon_source_set_size_wildcarded(isrc, TRUE);
    gtk_icon_source_set_state_wildcarded(isrc, TRUE);
    gtk_icon_source_set_direction_wildcarded(isrc, TRUE);
    gtk_icon_set_add_source(iset, isrc);
    gtk_icon_factory_add(ifac, GTK_STOCK_MEDIA_STOP, iset);
    gtk_icon_set_unref(iset);
    
    /* about icon */
    iset = gtk_icon_set_new();
    isrc = gtk_icon_source_new();
    pix = gdk_pixbuf_from_pixdata(&about_icon, FALSE, NULL);
    gtk_icon_source_set_pixbuf(isrc, pix);
    g_object_unref(G_OBJECT(pix));
    gtk_icon_source_set_size_wildcarded(isrc, TRUE);
    gtk_icon_source_set_state_wildcarded(isrc, TRUE);
    gtk_icon_source_set_direction_wildcarded(isrc, TRUE);
    gtk_icon_set_add_source(iset, isrc);
    gtk_icon_factory_add(ifac, GTK_STOCK_ABOUT, iset);
    gtk_icon_set_unref(iset);
#endif
}

static GdkPixbuf *
load_squished_icon(const gchar *stock_id, GtkIconSize size)
{
    GdkPixbuf *old_pix, *pix = NULL;
    GtkWidget *dummy;
    
    /* lame */
    dummy = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_widget_realize(dummy);
    
    old_pix = gtk_widget_render_icon(dummy, stock_id, size, NULL);
    
    gtk_widget_destroy(dummy);
    
    if(old_pix) {
        gint cur_w = gdk_pixbuf_get_width(old_pix);
        pix = gdk_pixbuf_scale_simple(old_pix, cur_w, (cur_w * 2) / 3,
                GDK_INTERP_BILINEAR);
        g_object_unref(G_OBJECT(old_pix));
    } else {
        DBG("couldn't render icon for '%s'", stock_id);
    }
    
    return pix;
}

static void
icon_theme_changed_stock_update(GtkIconTheme *icon_theme, gpointer user_data)
{
    GtkWidget *img = user_data;
    GdkPixbuf *pix;
    
    pix = load_squished_icon(g_object_get_data(G_OBJECT(img), "xfmedia-stock-id"),
            GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(img), "xfmedia-stock-size")));
    gtk_image_set_from_pixbuf(GTK_IMAGE(img), pix);
    g_object_unref(G_OBJECT(pix));
}

GtkWidget *
xfmedia_squished_stock_icon_load(const gchar *stock_id, GtkIconSize size)
{
    GdkPixbuf *pix;
    GtkWidget *img;
    
    pix = load_squished_icon(stock_id, size);
    
    img = gtk_image_new_from_pixbuf(pix);
    g_object_unref(G_OBJECT(pix));
    g_object_set_data(G_OBJECT(img), "xfmedia-stock-id", (gpointer)stock_id);
    g_object_set_data(G_OBJECT(img), "xfmedia-stock-size",
            GUINT_TO_POINTER(size));
    g_signal_connect(G_OBJECT(icon_theme), "changed",
            G_CALLBACK(icon_theme_changed_stock_update), img);
    
    return img;
}


/*************************************************
 * backports of gtk 2.4 stuff for gtk 2.2 compat *
 *************************************************/
    
#if !GLIB_CHECK_VERSION(2, 4, 0)
/* nicked from glib 2.4.5 */
GList *
g_queue_peek_nth_link (GQueue *queue,
                       guint   n)
{
  GList *link;
  gint i;

  g_return_val_if_fail (queue != NULL, NULL);

  if (n >= queue->length)
    return NULL;

  if (n > queue->length / 2)
    {
      n = queue->length - n - 1;

      link = queue->tail;
      for (i = 0; i < n; ++i)
        link = link->prev;
    }
  else
    {
      link = queue->head;
      for (i = 0; i < n; ++i)
        link = link->next;
    }

  return link;
}

void
g_queue_unlink (GQueue *queue,
                GList  *link_)
{
  g_return_if_fail (queue != NULL);
  g_return_if_fail (link_ != NULL);

  if (link_ == queue->tail)
    queue->tail = queue->tail->prev;
  
  queue->head = g_list_remove_link (queue->head, link_);
  queue->length--;
}

gpointer
g_queue_pop_nth (GQueue *queue,
                 guint   n)
{
  GList *nth_link;
  gpointer result;
  
  g_return_val_if_fail (queue != NULL, NULL);

  if (n >= queue->length)
    return NULL;
  
  nth_link = g_queue_peek_nth_link (queue, n);
  result = nth_link->data;
  
  g_queue_unlink (queue, nth_link);
  g_list_free (nth_link);

  return result;
}
#endif

#if !GTK_CHECK_VERSION(2, 4, 0)

#define _NET_WM_STATE_REMOVE 0
#define _NET_WM_STATE_ADD    1

static void
_xfmedia_change_netwm_state(GdkWindow *window, const gchar *state, gboolean add)
{
    XClientMessageEvent xevt;
    Display *dpy = GDK_WINDOW_XDISPLAY(window);
    GdkScreen *gscreen = gdk_drawable_get_screen(GDK_DRAWABLE(window));
    Window root = GDK_WINDOW_XWINDOW(gdk_screen_get_root_window(gscreen));
    
    xevt.type = ClientMessage;
    xevt.serial = 0;
    xevt.send_event = True;
    xevt.window = GDK_WINDOW_XWINDOW(window);
    xevt.message_type = XInternAtom(dpy, "_NET_WM_STATE", False);
    xevt.format = 32;
    xevt.data.l[0] = add ? _NET_WM_STATE_ADD : _NET_WM_STATE_REMOVE;
    xevt.data.l[1] = XInternAtom(dpy, state, False);
    xevt.data.l[2] = 0;
    
    gdk_error_trap_push();
    XSendEvent(dpy, root, False,
            SubstructureRedirectMask|SubstructureNotifyMask, (XEvent *)&xevt);
    gdk_error_trap_pop();
}


void
gtk_window_set_keep_above(GtkWindow *window, gboolean setting)
{
    GdkWindow *the_win = NULL;
    
    if(window->frame)
        the_win = window->frame;
    else if(GTK_WIDGET(window)->window)
        the_win = GTK_WIDGET(window)->window;
    else
        return;
    
    _xfmedia_change_netwm_state(the_win, "_NET_WM_STATE_ABOVE", setting);
}

void
gtk_window_set_accept_focus(GtkWindow *window, gboolean setting)
{
    XWMHints hints;
    GdkWindow *gwin = GTK_WIDGET(window)->window;
    
    hints.flags = InputHint;
    hints.input = setting ? True : False;
    
    XSetWMHints(GDK_WINDOW_XDISPLAY(gwin), GDK_WINDOW_XID(gwin), &hints);
}

#endif
